@@ -182,7 +182,10 @@ class QBzrCommand(Command):
                 if ret_code is None:
                     main_window = getattr(self, "main_window", None)
                     if main_window is not None:
-                        ret_code = getattr(main_window, "return_code", None)
+                        # 0.20 special: We check hasattr() first to work around
+                        # <http://bugs.python.org/issue4230>
+                        if hasattr(main_window, "return_code"):
+                            ret_code = main_window.return_code
                 return ret_code
             except Exception:
                 ui_mode = kwargs.get("ui_mode", False)
