@@ -52,24 +52,27 @@ from bzrlib.plugins.qbzr.lib.uifactory import ui_current_widget
 from bzrlib.plugins.qbzr.lib.trace import reports_exception
 from bzrlib.plugins.qbzr.lib.logwidget import LogList
 from bzrlib.plugins.qbzr.lib.decorators import lazy_call
+from bzrlib.plugins.qbzr.lib.widgets.texteditaccessory import setup_guidebar_for_find
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), '''
 from bzrlib import transform
 from bzrlib.workingtree import WorkingTree
 from bzrlib.plugins.qbzr.lib.encoding_selector import EncodingMenuSelector
-from bzrlib.plugins.qbzr.lib.diffwindow import DiffItem
+from bzrlib.plugins.qbzr.lib.diff import DiffItem
 from bzrlib.shelf import Unshelver
 from bzrlib.shelf_ui import Unshelver as Unshelver_ui
 from bzrlib.plugins.qbzr.lib.subprocess import SimpleSubProcessDialog
+import sip
 ''')
 
+
 class ShelveListWidget(ToolbarPanel):
 
     def __init__(self, directory=None, complete=False, ignore_whitespace=False, 
                  encoding=None, splitters=None, parent=None):
         ToolbarPanel.__init__(self, slender=False, icon_size=22, parent=parent)
 
-        self.encoding = encoding
+        self.initial_encoding = encoding
         self.directory = directory
 
         self.current_diffs = []
@@ -120,7 +123,7 @@ class ShelveListWidget(ToolbarPanel):
         self.tabwidth_selector = TabWidthMenuSelector(label_text=gettext("Tab width"), 
                                     onChanged=self.on_tabwidth_changed)
         view_menu.addMenu(self.tabwidth_selector)
-        self.encoding_selector = EncodingMenuSelector(self.encoding,
+        self.encoding_selector = EncodingMenuSelector(encoding,
                                     gettext("Encoding"), self.encoding_changed)
         self.encoding_selector.setIcon(get_icon("format-text-bold", 16))
         view_menu.addMenu(self.encoding_selector)
@@ -128,10 +131,13 @@ class ShelveListWidget(ToolbarPanel):
                 N_("&View Options"), view_menu, icon_name="document-properties",
                 shortcut="Alt+V")
 
-        self.find_toolbar = FindToolbar(self, self.diffviews[0].browsers[0], show_find)
+        self.find_toolbar = FindToolbar(self, self.diffviews[0].browsers, show_find)
         diff_panel.add_widget(self.find_toolbar)
         diff_panel.add_widget(self.stack)
         self.find_toolbar.hide()
+        for gb in self.diffviews[0].guidebar_panels:
+            setup_guidebar_for_find(gb, self.find_toolbar, 1)
+        setup_guidebar_for_find(self.diffviews[1], self.find_toolbar, 1)
 
         # Layout widgets
         self.splitter1 = QtGui.QSplitter(QtCore.Qt.Horizontal)
@@ -273,7 +279,6 @@ class ShelveListWidget(ToolbarPanel):
             manager = tree.get_shelf_manager()
             shelves = manager.active_shelves()
             for shelf_id in reversed(shelves):
-
                 message = manager.get_metadata(shelf_id).get('message')
                 item = QtGui.QTreeWidgetItem()
                 item.setText(0, unicode(shelf_id))
@@ -284,7 +289,12 @@ class ShelveListWidget(ToolbarPanel):
             self.tree = tree
             self.manager = manager
 
-            tabwidth = get_set_tab_width_chars(tree.branch)
+            branch = tree.branch
+            if self.initial_encoding is None:
+                encoding = get_set_encoding(None, branch)
+                self.initial_encoding = encoding            # save real encoding for the next time
+                self.encoding_selector.encoding = encoding  # set encoding selector
+            tabwidth = get_set_tab_width_chars(branch)
             self.tabwidth_selector.setTabWidth(tabwidth)
             self._on_tabwidth_changed(tabwidth)
 
@@ -328,19 +338,8 @@ class ShelveListWidget(ToolbarPanel):
 
     def load_diff(self, tree, base_tree):
         self.file_view.clear()
-
-        changes = tree.iter_changes(base_tree)
-
-        def changes_key(change):
-            return change[1][1] or change[1][0]
         
-        for (file_id, paths, changed_content, versioned, parent, 
-                name, kind, executable) in sorted(changes, key=changes_key):
-            di = DiffItem.create([base_tree, tree], file_id, paths, changed_content,
-                    versioned, parent, name, kind, executable)
-            if not di:
-                continue
-
+        for di in DiffItem.iter_items((base_tree, tree), lock_trees=False):
             di.load()
 
             old_path, new_path = di.paths
@@ -370,6 +369,8 @@ class ShelveListWidget(ToolbarPanel):
     @lazy_call(100, per_instance=True)
     @runs_in_loading_queue
     def _show_selected_diff(self):
+        if sip.isdeleted(self):
+            return
         self._interrupt_switch = False 
         try:
             refresh = self._need_refresh
@@ -383,6 +384,7 @@ class ShelveListWidget(ToolbarPanel):
                 appends = diffs[cur_len:]
             else:
                 for view in self.diffviews:
+                    view.set_complete(self.complete)
                     view.clear()
                 self.current_diffs = []
                 appends = diffs 
@@ -390,8 +392,9 @@ class ShelveListWidget(ToolbarPanel):
                 lines = d.lines
                 groups = d.groups(self.complete, self.ignore_whitespace)
                 dates = d.dates[:]  # dates will be changed in append_diff
-                ulines = d.encode((self.encoding_selector.encoding,
-                                   self.encoding_selector.encoding))
+                ulines = d.get_unicode_lines(
+                    (self.encoding_selector.encoding,
+                     self.encoding_selector.encoding))
                 data = [''.join(l) for l in ulines]
                 for view in self.diffviews:
                     view.append_diff(list(d.paths), d.file_id, d.kind, d.status, dates, 
@@ -403,6 +406,8 @@ class ShelveListWidget(ToolbarPanel):
                     break
         finally:
             self._interrupt_switch = False
+            for view in self.diffviews[0].browsers + (self.diffviews[1],):
+                view.emit(QtCore.SIGNAL("documentChangeFinished()"))
 
     def selected_shelve_changed(self):
         self._change_current_shelve()
@@ -410,6 +415,8 @@ class ShelveListWidget(ToolbarPanel):
     @lazy_call(100, per_instance=True)
     @runs_in_loading_queue
     def _change_current_shelve(self):
+        if sip.isdeleted(self):
+            return
         items = self.shelve_view.selectedItems()
         if len(items) != 1:
             self.shelf_id = None
@@ -449,11 +456,11 @@ class ShelveListWidget(ToolbarPanel):
     def unidiff_toggled(self, state):
         index = 1 if state else 0
         self.diffviews[index].rewind()
-        self.stack.setCurrentIndex(index)
         if index == 0:
-            self.find_toolbar.text_edit = self.diffviews[0].browsers[0]
+            self.find_toolbar.set_text_edits(self.diffviews[0].browsers)
         else:
-            self.find_toolbar.text_edit = self.diffviews[1]
+            self.find_toolbar.set_text_edits([self.diffviews[1].view])
+        self.stack.setCurrentIndex(index)
 
     def complete_toggled(self, state):
         self.complete = state
@@ -518,7 +525,7 @@ class ShelveListWidget(ToolbarPanel):
     def eventFilter(self, object, event):
         if event.type() == QtCore.QEvent.FocusIn:
             if object in self.diffviews[0].browsers:
-                self.find_toolbar.text_edit = object
+                self.find_toolbar.set_text_edit(object)
         return ToolbarPanel.eventFilter(self, object, event)
     
     def load_settings(self):
